@@ -253,6 +253,9 @@ gpm_brightness_update_hw (GpmBrightness *brightness)
 	gboolean retval;
 	gint     brightness_hw = 0;
 
+	g_return_val_if_fail (brightness != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
+
 	retval = TRUE;
 	if (!dbus_g_proxy_call (brightness->priv->proxy, "GetBrightness", &error,
 				G_TYPE_INVALID,
@@ -285,6 +288,9 @@ gpm_brightness_set_hw (GpmBrightness *brightness,
 	gint     ret;
 	gboolean retval;
 
+	g_return_val_if_fail (brightness != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
+
 	if (brightness_level_hw < 0 ||
 	    brightness_level_hw > brightness->priv->levels - 1) {
 		gpm_warning ("set outside range (%i of %i)",
@@ -340,16 +346,20 @@ gpm_brightness_percent_to_hw (int percentage,
  * gpm_brightness_dim_hw:
  * @brightness: This brightness class instance
  * @new_level_hw: The new hardware level
+ * Return value: Success.
  *
  * Just do the step up and down, after knowing the step interval
  **/
-static void
+static gboolean
 gpm_brightness_dim_hw_step (GpmBrightness *brightness,
 		            int            new_level_hw,
 		            int		   step_interval)
 {
 	int current_hw;
 	int a;
+
+	g_return_val_if_fail (brightness != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
 	current_hw = brightness->priv->current_hw;
 
 	/* we do the step interval as we can have insane levels of brightness */
@@ -366,17 +376,23 @@ gpm_brightness_dim_hw_step (GpmBrightness *brightness,
 			g_usleep (1000 * DIM_INTERVAL);
 		}
 	}
+	return TRUE;
 }
 
 /**
  * gpm_brightness_get_step:
  * @brightness: This brightness class instance
- * Return value: the amount of hardware steps to do on each update
+ * Return value: the amount of hardware steps to do on each update or
+ * zero for error.
  **/
 static int
 gpm_brightness_get_step (GpmBrightness *brightness)
 {
 	int step;
+
+	g_return_val_if_fail (brightness != NULL, 0);
+	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), 0);
+
 	if (brightness->priv->levels < 20) {
 		/* less than 20 states should do every state */
 		step = 1;
@@ -391,26 +407,31 @@ gpm_brightness_get_step (GpmBrightness *brightness)
  * gpm_brightness_dim_hw:
  * @brightness: This brightness class instance
  * @new_level_hw: The new hardware level
+ * Return value: Success.
  **/
-static void
+static gboolean
 gpm_brightness_dim_hw (GpmBrightness *brightness,
 		       int	      new_level_hw)
 {
 	int step;
 
+	g_return_val_if_fail (brightness != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
+
 	if (! brightness->priv->has_hardware) {
-		return;
+		return FALSE;
 	}
 
 	/* some machines don't take kindly to auto-dimming */
 	if (brightness->priv->does_own_dimming) {
 		gpm_brightness_set_hw (brightness, new_level_hw);
-		return;
+		return TRUE;
 	}
 
 	/* macbook pro has a bazzillion brightness levels, be a bit clever */
 	step = gpm_brightness_get_step (brightness);
 	gpm_brightness_dim_hw_step (brightness, new_level_hw, step);
+	return TRUE;
 }
 
 /**
@@ -439,12 +460,17 @@ gpm_brightness_hw_to_percent (int hw,
  * gpm_brightness_set_level_dim:
  * @brightness: This brightness class instance
  * @brightness_level: The percentage brightness
+ * Return value: Success.
  **/
-void
+gboolean
 gpm_brightness_set_level_dim (GpmBrightness *brightness,
 			      int	     brightness_level)
 {
 	int level_hw;
+
+	g_return_val_if_fail (brightness != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
+
 	level_hw = gpm_brightness_percent_to_hw (brightness_level, brightness->priv->levels);
 
 	/* If the current brightness is less than the dim brightness then just
@@ -457,61 +483,83 @@ gpm_brightness_set_level_dim (GpmBrightness *brightness,
 			     brightness->priv->level_std_hw, level_hw);
 		brightness->priv->level_dim_hw = brightness->priv->level_std_hw;
 	}
+	return TRUE;
 }
 
 /**
  * gpm_brightness_set_level_dim:
  * @brightness: This brightness class instance
  * @brightness_level: The percentage brightness
+ * Return value: Success.
  **/
-void
+gboolean
 gpm_brightness_set_level_std (GpmBrightness *brightness,
 			      int	     brightness_level)
 {
 	int level_hw;
+
+	g_return_val_if_fail (brightness != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
+
 	level_hw = gpm_brightness_percent_to_hw (brightness_level,
 						 brightness->priv->levels);
 	brightness->priv->level_std_hw = level_hw;
+	return TRUE;
 }
 
 /**
  * gpm_brightness_dim:
  * @brightness: This brightness class instance
+ * Return value: Success.
  *
  * Sets the screen into dim mode, where the dim brightness is used.
  **/
-void
+gboolean
 gpm_brightness_dim (GpmBrightness *brightness)
 {
+	g_return_val_if_fail (brightness != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
+
 	gpm_brightness_dim_hw (brightness, brightness->priv->level_dim_hw);
+	return TRUE;
 }
 
 /**
  * gpm_brightness_undim:
  * @brightness: This brightness class instance
+ * Return value: Success.
  *
  * Sets the screen into normal mode, where the startdard brightness is used.
  **/
-void
+gboolean
 gpm_brightness_undim (GpmBrightness *brightness)
 {
+	g_return_val_if_fail (brightness != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
+
 	gpm_brightness_dim_hw (brightness, brightness->priv->level_std_hw);
+	return TRUE;
 }
 
 /**
  * gpm_brightness_set:
  * @brightness: This brightness class instance
+ * Return value: Success.
  **/
-void
+gboolean
 gpm_brightness_set (GpmBrightness *brightness)
 {
+	g_return_val_if_fail (brightness != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
+
 	gpm_brightness_dim_hw (brightness, brightness->priv->level_std_hw);
+	return TRUE;
 }
 
 /**
  * gpm_brightness_get:
  * @brightness: This brightness class instance
- * Return value: The percentage brightness, or -1 for no hardware
+ * Return value: The percentage brightness, or -1 for no hardware or error
  *
  * Gets the current (or at least what this class thinks is current) percentage
  * brightness. This is quick as no HAL inquiry is done.
@@ -520,6 +568,10 @@ int
 gpm_brightness_get (GpmBrightness *brightness)
 {
 	int percentage;
+
+	g_return_val_if_fail (brightness != NULL, -1);
+	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), -1);
+
 	if (! brightness->priv->has_hardware) {
 		return -1;
 	}
@@ -531,16 +583,21 @@ gpm_brightness_get (GpmBrightness *brightness)
 /**
  * gpm_brightness_up:
  * @brightness: This brightness class instance
+ * Return value: Success.
  *
  * If possible, put the brightness of the LCD up one unit.
  **/
-void
+gboolean
 gpm_brightness_up (GpmBrightness *brightness)
 {
 	int step;
+	int percentage;
+
+	g_return_val_if_fail (brightness != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
 
 	if (! brightness->priv->has_hardware) {
-		return;
+		return FALSE;
 	}
 
 	/* Do we find the new value, or set the new value */
@@ -552,26 +609,31 @@ gpm_brightness_up (GpmBrightness *brightness)
 		gpm_brightness_set_hw (brightness, brightness->priv->current_hw + step);
 	}
 
-	int percentage;
 	percentage = gpm_brightness_hw_to_percent (brightness->priv->current_hw,
 						   brightness->priv->levels);
 	gpm_debug ("emitting brightness-step-changed : %i", percentage);
 	g_signal_emit (brightness, signals [BRIGHTNESS_STEP_CHANGED], 0, percentage);
+	return TRUE;
 }
 
 /**
  * gpm_brightness_down:
  * @brightness: This brightness class instance
+ * Return value: Success.
  *
  * If possible, put the brightness of the LCD down one unit.
  **/
-void
+gboolean
 gpm_brightness_down (GpmBrightness *brightness)
 {
 	int step;
+	int percentage;
+
+	g_return_val_if_fail (brightness != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
 
 	if (! brightness->priv->has_hardware) {
-		return;
+		return FALSE;
 	}
 
 	/* Do we find the new value, or set the new value */
@@ -583,9 +645,25 @@ gpm_brightness_down (GpmBrightness *brightness)
 		gpm_brightness_set_hw (brightness, brightness->priv->current_hw - 1);
 	}
 
-	int percentage;
 	percentage = gpm_brightness_hw_to_percent (brightness->priv->current_hw,
 						   brightness->priv->levels);
 	gpm_debug ("emitting brightness-step-changed : %i", percentage);
 	g_signal_emit (brightness, signals [BRIGHTNESS_STEP_CHANGED], 0, percentage);
+	return TRUE;
+}
+
+/**
+ * gpm_brightness_has_hardware:
+ * @brightness: This brightness class instance
+ * Return value: if we can set or get the lcd brightness
+ *
+ * Returns if the computer has brightness hardware (lcd panel)
+ **/
+gboolean
+gpm_brightness_has_hardware (GpmBrightness *brightness)
+{
+	g_return_val_if_fail (brightness != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
+
+	return brightness->priv->has_hardware;
 }
